/*
1. LCD connection - library: LiquidCrystal.h
 Connection:
 =================================
 LCD pin              Connect to
 ---------------------------------
 01 - GND             GND, pot
 02 - VCC             +5V, pot
 03 - Contrast        Pot wiper
 04 - RS              P1.0
 05 - R/W             GND
 06 - EN              P1.1
 07 - DB0             GND
 08 - DB1             GND
 09 - DB2             GND
 10 - DB3             GND
 11 - DB4             P1.2
 12 - DB5             P1.3
 13 - DB6             P1.4
 14 - DB7             P1.5
 15 - BL+             +5V
 16 - BL-             GND
 =================================

2. I2C Bus - Library: Wire.h
   Connection:
      SCL := P2.1
      SDA := P2.2
 */

// include the library code:
#include <LiquidCrystal.h>
#include <Wire.h>

// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(P1_0, P1_1, P1_2, P1_3, P1_4, P1_5);

// --------------------------------------------------------------------
byte v_byte;

byte in_kbd_7;
byte in_kbd_6;
byte in_kbd_5;
byte in_kbd_4;

byte active_key;
byte act_key_ascii;
byte gv_refresh = 0;
byte i2c_add;

byte v_temp_msb;
byte v_temp_lsb;

byte temp_res;
byte v_config_msb;
byte gv_format;
byte i2c_answer;

byte rtc_sec;
byte rtc_min;
byte rtc_hrs;
byte rtc_day;
byte rtc_mnt;
int  rtc_year;
byte rtc_wday;

byte gv_exit;

byte gv_hyt939 = 0xFF;
byte v_h939_cap_msb;
byte v_h939_cap_lsb;
float h939_rf;
float h939_rf_result = 0;
int h939_rf_old;
int h939_rf_new;
byte lv_c2 = 0;

byte v_h939_t_msb;
byte v_h939_t_lsb;
float h939_temp;
float h939_temp_result = 0;

int v_h939_c_raw;
int v_h939_t_raw;
long h939_temp_long;
int h939_t_old;
int h939_t_new;
byte lv_c3 = 0;
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void setup() {
  // set up the LCD's number of columns and rows: 
 
  lcd.begin(20, 4);

  lcd.setCursor(0,0);
  lcd.print("MSP430 Hygrometer");
  lcd.setCursor(0,1);
  lcd.print("RTC & Thermometer");
  lcd.setCursor(0,2);
  lcd.print("Energia v1.14.38");
  lcd.setCursor(0,3);
  lcd.print("12.04.2021");

  delay(3000);
  lcd.clear();

  Wire.begin();
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void loop()
{
  i2c_add = 0b01001000;
  entry_main();
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// ------------ Main Selection Screen / Entry Screen ------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void entry_main()
{
  main_app();
  
  while(1)
  {
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("(1) AT30TSE754");
    lcd.setCursor(0,1);
    lcd.print("(2) RTC: DS1307");           //or  M41T00CAP
    lcd.setCursor(0,2);
    lcd.print("(3) HYT939");

    lcd.setCursor(0,3);
    lcd.print("(9) Main Application");
    gv_refresh = 0;

    while(gv_refresh == 0)
    {
      read_kbd();
      switch(active_key)
      {
        case 0x01: i2c_temp_1(); break;
        case 0x02: i2c_datum_1(); break;
        case 0x03: hyt939(); break;
        case 0x09: main_app(); break;
      }
    }
  }
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --- Main application: RTC + Temperature
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void main_app()
{
   byte temp_lsb_old = 0xFE;
   byte temp_lsb_new;
   byte lv_c1;
  
   lcd.clear();
   w_rtc_txt(1);
   
   gv_format = 1;           //display 2 decimal places
   
   while(active_key != 0xFF) read_kbd();
   while(active_key != 0x0E)
   {
//   ---- RTC -----------------------------------------
     i2c_rtc_read();
     if (i2c_answer == 0)
     {
       w_date_time(3);
     }
     else
     {
       lcd.setCursor(6,0);
       lcd.print("--           ");
       lcd.setCursor(6,1);
       lcd.print("--           ");
       lcd.setCursor(6,3);
       lcd.print("--           ");
     }
//   ---- HYT939 ------------------------------------
     gv_hyt939 = hyt939_start();
     lcd.setCursor(14,2);
     if(gv_hyt939 == 0)
     {
       hyt939_read();
       lcd.setCursor(14,2);
       lcd.print(" ");
       if(h939_rf_result < 10) lcd.print(" ");
       lcd.print(h939_rf_result, 1);
       lcd.print("%");
     }
     else lcd.print("      ");
     w_rtc_txt(1);
//   ---- AT30TSE754 --------------------------------
     find_i2c_temp();         //find attached I2C sensor
     temp_res = 1;
     t_change_res();          //set resolution to 11 bit
     Wire.beginTransmission(i2c_add);
     Wire.write(0x00);
     i2c_answer = Wire.endTransmission(1);
     if (i2c_answer == 0)
     {
       Wire.requestFrom(i2c_add, 2, 1);    //I2C address / 2 bytes // stop
       v_temp_msb =  Wire.read();
       temp_lsb_new =  Wire.read();
       if(temp_lsb_old == 0xFE)
       {
          v_temp_lsb = temp_lsb_new;
          temp_lsb_old = temp_lsb_new;
          lv_c1 = 0;
       }
       if(temp_lsb_new != temp_lsb_old) lv_c1++;
       else lv_c1 = 0;
       if(lv_c1 == 3)
       {
          v_temp_lsb = temp_lsb_new;
          temp_lsb_old = temp_lsb_new;
          lv_c1 = 0;
       }
       w_temp_ub(6,2,1);
     }
     else
     {
        if(gv_hyt939 == 0)
        {
          lcd.setCursor(6,2);
          if(h939_temp_result>=0)
          {
            lcd.print("+");
            if(h939_temp_result < 10) lcd.print("0");
          }
          lcd.print(h939_temp_result,2);
          lcd.print(char(223));
          lcd.print("C");
        }
        else
        {
          v_temp_lsb = 0xFF;
          w_temp_ub(6,2,1);
        }
     }
     
     read_kbd();
     delay(100);
   }
   gv_refresh = 1;
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void find_i2c_temp()
{
   i2c_add = 0b01001000;

   while(1)
   {
      Wire.beginTransmission(i2c_add);
      Wire.write(0x00);
      i2c_answer = Wire.endTransmission(1);
      if (i2c_answer == 0) return;
      
      i2c_add++;
      if (bitRead(i2c_add,3) == 0)
      {
        i2c_add = 0b01001000;
        return;
      }
   }
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void w_rtc_txt(byte x1)
{
  lcd.setCursor(0,0);
  lcd.print("Date: ");
  lcd.setCursor(0,1);
  lcd.print("Day : ");
  lcd.setCursor(0,3);
  lcd.print("Time:");
  if(x1 == 0)
  {

    lcd.setCursor(17,3);
    lcd.print("(D)");
  }
  if(x1 == 1)
  {
    lcd.setCursor(0,2);
    if(gv_hyt939 == 0) lcd.print("T/H :");
    else lcd.print("Temp:");
  }
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// ---- I2C RTC
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void i2c_datum_1()
{  
   lcd.clear();
   w_rtc_txt(0);
   
   while(active_key != 0xFF) read_kbd();
   while(active_key != 0x0E)
   {
      w_rtc_txt(0);
      i2c_rtc_read();
      if (i2c_answer == 0)
      {  
        w_date_time(3);
      }
      else
      {
        lcd.setCursor(0,0);
        lcd.print("No answer           ");
        lcd.setCursor(0,1);
        lcd.print("from RTC Chip       ");
        lcd.setCursor(0,3);
        lcd.print("                    ");
      }
      read_kbd();
      delay(100);
      if (active_key == 0x0D && i2c_answer == 0)
      {
      rtc_setup();
      lcd.setCursor(0,2);
      lcd.print("                    ");
      lcd.setCursor(17,3);
      lcd.print("(D)");
      }
   }
   gv_refresh = 1;
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void w_date_time(byte w_col)
{
  rtc_day = ( ( rtc_day & 0b11110000 ) / 16 ) * 10 + ( rtc_day & 0b00001111 );
  rtc_mnt = ( ( rtc_mnt & 0b11110000 ) / 16 ) * 10 + ( rtc_mnt & 0b00001111 );
  rtc_year = ( ( rtc_year & 0b11110000 ) / 16 ) * 10 + ( rtc_year & 0b00001111 );
  rtc_year = rtc_year + 2000;
        
  lcd.setCursor(6,0);
  w_print_dec01_wlz(rtc_day);
  lcd.print(".");
  w_print_dec01_wlz(rtc_mnt);
  lcd.print(".");
  lcd.print(rtc_year);

  calc_day();

  lcd.setCursor(6,1);
  w_week_day(0);
  lcd.print("   ");

  lcd.setCursor(6,w_col);
  rtc_hrs = ( ( rtc_hrs & 0b00110000 ) / 16 ) * 10 + ( rtc_hrs & 0b00001111 );
  w_print_dec01_wlz(rtc_hrs);
  lcd.print(":");
  rtc_min = ( ( rtc_min & 0b01110000 ) / 16 ) * 10 + ( rtc_min & 0b00001111 );
  w_print_dec01_wlz(rtc_min);
  lcd.print(":");
  rtc_sec = ( ( rtc_sec & 0b01110000 ) / 16 ) * 10 + ( rtc_sec & 0b00001111 );
  w_print_dec01_wlz(rtc_sec);
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --- I2C RTC Setup --------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void rtc_setup()
{
  lcd.setCursor(0,2);
  lcd.print("<Save:B/ESC:*/NXT:#>");
  lcd.setCursor(17,3);
  lcd.print("   ");

  gv_exit = 0;
  
  while(gv_exit == 0)
  {
    rtc_day = get_k_number(6,0, rtc_day, 31); if (rtc_day == 0xFF) return;
    rtc_mnt = get_k_number(9,0, rtc_mnt, 12); if (rtc_mnt == 0xFF) return;
    rtc_year = get_k_number(14,0, rtc_year - 2000, 99) + 2000; if (rtc_year == 2255) return;    
  }
  if(gv_exit == 1) gv_exit = 0;
  while(gv_exit == 0)
  {
    rtc_hrs = get_k_number(6,3, rtc_hrs, 23); if (rtc_hrs == 0xFF) return;
    rtc_min = get_k_number(9,3, rtc_min, 59); if (rtc_min == 0xFF) return;
    rtc_sec = get_k_number(12,3, rtc_sec, 59); if (rtc_sec == 0xFF) return;    
  }
  if (gv_exit == 2)
  {
    //Save
    i2c_rtc_write();
    lcd.setCursor(0,2);
    lcd.print("Changes Saved.......");
    delay(1000);
    lcd.setCursor(0,2);
    lcd.print("                    ");
  }
}

// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --- read two digits (number) from the keyboard
// --------------------------------------------------------------------
// --------------------------------------------------------------------
byte get_k_number(byte i_col, byte i_row, byte i_value, byte i_max)
{ 
  byte lv_blink;
  byte lv_pos = 0;
  byte lv_msb;
  byte lv_lsb;

  if(gv_exit == 2) return(i_value);
  gv_exit = 0;

  lv_lsb = i_value % 10;
  lv_msb = i_value / 10;
  
  while(active_key != 0xFF) read_kbd();
  while(1)
   {
      read_kbd();
      if (active_key == 0x0E)
      {
        while(active_key != 0xFF) read_kbd();
        return(0xFF);                            //Escape
      }
      if(active_key == 0x0F)
      {
        lcd.setCursor(i_col,i_row);
        w_print_dec01_wlz(i_value);
        gv_exit = 1;
        return(i_value);                        //Next
      }
      if(active_key == 0x0B)
      {
        lcd.setCursor(i_col,i_row);
        w_print_dec01_wlz(i_value);
        gv_exit = 2;
        return(i_value);                        //Save Request
      }
      if(active_key < 10)
      {
        //key between 0 and 8 has been pressed
        if(lv_pos == 0) lv_msb = active_key;
        if(lv_pos == 1) lv_lsb = active_key;
        i_value = lv_msb * 10 + lv_lsb;
        if(i_value > i_max)
        {
          lv_pos = 1;
          i_value = i_max;
          lv_lsb = i_value % 10;
          lv_msb = i_value / 10;
        }
        lv_pos++;
        if(lv_pos == 2)
        {
          lv_pos = 0;
          if(i_max == 31) rtc_day = i_value;
          if(i_max == 12) rtc_mnt = i_value;
          if(i_max == 99) rtc_year = i_value + 2000;
          calc_day();
          lcd.setCursor(6,1);
          w_week_day(0);
          lcd.print("   ");
          lcd.setCursor(i_col,i_row);
          w_print_dec01_wlz(i_value);
          return(i_value);
        }
        lcd.setCursor(i_col,i_row);
        w_print_dec01_wlz(i_value);
        while(active_key != 0xFF) read_kbd();
        lv_blink = 30;
      }
      
      lcd.setCursor(i_col,i_row);
      if(lv_blink > 20) w_print_dec01_wlz(i_value);
      else lcd.print("  ");
      lv_blink++;
      if(lv_blink > 40) lv_blink = 0;
      delay(20);
   }
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --- I2C RTC Write ---------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void i2c_rtc_write()
{
   rtc_sec = (rtc_sec/10)*16 + (rtc_sec%10);
   rtc_min = (rtc_min/10)*16 + (rtc_min%10);
   rtc_hrs = (rtc_hrs/10)*16 + (rtc_hrs%10);
   rtc_day = (rtc_day/10)*16 + (rtc_day%10);
   rtc_mnt = (rtc_mnt/10)*16 + (rtc_mnt%10);
   rtc_year = rtc_year - 2000;
   rtc_year = (rtc_year/10)*16 + (rtc_year%10);
   
   Wire.beginTransmission(0b01101000);
   Wire.write(0x00);
   Wire.write(rtc_sec);
   Wire.write(rtc_min);
   Wire.write(rtc_hrs);
   Wire.write(rtc_wday);
   Wire.write(rtc_day);
   Wire.write(rtc_mnt);
   Wire.write(rtc_year);
   i2c_answer = Wire.endTransmission(1);
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --- I2C RTC Read ---------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void i2c_rtc_read()
{
   Wire.beginTransmission(0b01101000);
   Wire.write(0x00);
   i2c_answer = Wire.endTransmission(1);
   
   Wire.requestFrom(0b01101000, 7, 1);    //I2C address / 2 bytes // stop
   rtc_sec = Wire.read();
   rtc_min = Wire.read();
   rtc_hrs = Wire.read();
   rtc_day = Wire.read();         //dummy read (day of a week)
   rtc_day = Wire.read();
   rtc_mnt = Wire.read();
   rtc_year = Wire.read();
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --- write day in a week
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void w_week_day(byte i_language)
{
   switch(rtc_wday)
     {
      case 1:
         lcd.print("Monday");
         break;
      case 2:
         lcd.print("Tuesday");
         break;
      case 3:
         lcd.print("Wednesday");
         break;
      case 4:
         lcd.print("Thursday");
         break;
      case 5:
         lcd.print("Friday");
         break;
      case 6:
         lcd.print("Saturday");
         break;
      case 0:
         lcd.print("Sunday");
         break;
     }
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --- Calculation of week day; only if date > 2000
//     https://de.wikipedia.org/wiki/Wochentagsberechnung
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void calc_day()
{
  byte cd_d;
  byte cd_m;
  byte cd_y;
  byte cd_c;
  word cd_help1;
  byte cd_help2;
  byte cd_help3;
  short int cd_w_w2;

  cd_d = rtc_day;
  cd_m = rtc_mnt - 2;
  if (rtc_mnt == 1) cd_m = 11;
  if (rtc_mnt == 2) cd_m = 12;
  cd_y = rtc_year - 2000;
  if (rtc_mnt < 3) cd_y = cd_y - 1;
  cd_c = 20;
  if ((rtc_mnt < 3) && (rtc_year == 2000)) cd_c = cd_c - 1;

//  cd_help1 = (2.6 * cd_m) - 0.2;
  cd_help1 = ((26 * cd_m) - 2) / 10;
  cd_help2 = cd_y / 4;
  cd_help3 = cd_c / 4;

  cd_w_w2 = (cd_d + cd_help1 + cd_y + cd_help2 + cd_help3 - 2 * cd_c);
  cd_w_w2 = cd_w_w2 % 7;

  if (cd_w_w2 < 0) cd_w_w2 = 7 + cd_w_w2;

  rtc_wday = cd_w_w2;
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------

// --------------------------------------------------------------------
// --------------------------------------------------------------------
// ---- I2C Temperature meassurement / AT30TSE754
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void hyt939()
{
   lcd.clear();  
   while(active_key != 0xFF) read_kbd();
   while(active_key != 0x0E)
   {
      i2c_answer = hyt939_start();
      if (i2c_answer == 0)
      {
        hyt939_read();
        lcd.setCursor(0,0);
        lcd.print("Humidity reg: ");
        w_hex(v_h939_cap_msb);
        lcd.print("-");
        w_hex(v_h939_cap_lsb);

        lcd.setCursor(0,3);
        lcd.print("Humidity: ");
        lcd.print(h939_rf, 1);
        lcd.print("%  ");
//      --------------------------------------------
        lcd.setCursor(0,1);
        lcd.print("Temp reg    : ");
        w_hex(v_h939_t_msb);
        lcd.print("-");
        w_hex(v_h939_t_lsb);
      }
      else
      {
        lcd.setCursor(0,0);
        lcd.print("HYT-sensor not found");
      }
//    -----------------------------------------------------------------
//    --- read the keypad ---------------------------------------------
//    -----------------------------------------------------------------
      read_kbd();
      delay(100);
   }
   gv_refresh = 1;
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
byte hyt939_start()
{
   Wire.beginTransmission(0b00101000);     //I2C Address = 0101 000x = 50h
   return(Wire.endTransmission(1));
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void hyt939_read()
{
// -------------------------------------------------------------------
// -- Read the Humidity and the Temperature --------------------------
// -------------------------------------------------------------------
   Wire.requestFrom(0b00101000, 4, 1);    //I2C address / 2 bytes // stop
   v_h939_cap_msb =  Wire.read();
   v_h939_cap_lsb =  Wire.read();
   v_h939_t_msb =  Wire.read();
   v_h939_t_lsb =  Wire.read();
// -------------------------------------------------------------------
// -- Calculate the Humidity -----------------------------------------
// -------------------------------------------------------------------
   v_h939_cap_msb = v_h939_cap_msb & 0b00111111;
   v_h939_c_raw = v_h939_cap_msb * 256 + v_h939_cap_lsb;
  
   h939_rf = (float)((float)v_h939_c_raw / 16383 ) * 100;
   h939_rf_new = (float)h939_rf * 10;

   if(h939_rf_new != h939_rf_old) lv_c2++;
   else lv_c2 = 0;
   if(lv_c2 == 3)
   {
      h939_rf_result = h939_rf;
      h939_rf_old = h939_rf_new;
      lv_c2 = 0;
   }   
   if (h939_rf_result == 0)
   {
     h939_rf_result = h939_rf;
     h939_rf_old = (float)h939_rf * 10;
   }
// -------------------------------------------------------------------
// -- Calculate the Temperature --------------------------------------
// -------------------------------------------------------------------
   v_h939_t_lsb = v_h939_t_lsb & 0b11111100;
   v_h939_t_raw = (v_h939_t_msb * 64) + (v_h939_t_lsb / 4);

   h939_temp_long = 165 * (long)v_h939_t_raw;

   h939_temp = (float)((float)h939_temp_long/16383) - 40;
   h939_t_new = (float)h939_temp * 10;
   if(h939_t_new != h939_t_old) lv_c3++;
   else lv_c3 = 0;
   if(lv_c3 == 3)
   {
      h939_temp_result = h939_temp;
      h939_t_old = h939_t_new;
      lv_c3 = 0;
   }   
   if (h939_temp_result == 0)
   {
     h939_temp_result = h939_temp;
     h939_t_old = (float)h939_temp * 10;
   }
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// ---- I2C Temperature meassurement / AT30TSE754
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void i2c_temp_1()
{
   gv_format = 0;
  
   lcd.clear();
   lcd.setCursor(0,0);
   lcd.print("AT30TSE754       (*)");
   lcd.setCursor(0,1);
   lcd.print("I2C Ad:          (A)");
   lcd.setCursor(0,2);
   lcd.print("Temp:            (2)");
   lcd.setCursor(0,3);
   lcd.print("Res :        bit (1)");
   
   while(active_key != 0xFF) read_kbd();
   
   while(active_key != 0x0E)
   {
      Wire.beginTransmission(i2c_add);
      Wire.write(0x00);
      i2c_answer = Wire.endTransmission(1);
      if (i2c_answer == 0)
      {
        lcd.setCursor(0,2);
        lcd.print("Temp: ");
        lcd.setCursor(17,2);
        lcd.print("(2)");
        lcd.setCursor(0,3);
        lcd.print("Res :     ");
        lcd.setCursor(13,3);
        lcd.print("bit (1)");
        Wire.requestFrom(i2c_add, 2, 1);    //I2C address / 2 bytes // stop
        v_temp_msb = 0xFF;
        v_temp_lsb = 0xFF;
        v_temp_msb =  Wire.read();
        v_temp_lsb =  Wire.read();
        w_temp_ub(6,2,0);
//      -----------------------------------------------------------------
//      --- configuration register: resolution --------------------------
//      -----------------------------------------------------------------
        read_t_config_reg();
        if (active_key == 0x01)
        {
          t_change_res();
          read_t_config_reg();
          while(active_key != 0xFF) read_kbd();
        }
        lcd.setCursor(10,3);
        temp_res = temp_res + 9;
        w_print_dec1_nlz(temp_res);
      }
      else
      {
        lcd.setCursor(0,2);
        lcd.print("No response         ");
        lcd.setCursor(0,3);
        lcd.print("from Sensor         ");
      }
      w_i2c_add(8,1);
      
//    -----------------------------------------------------------------
//    --- read the keypad ---------------------------------------------
//    -----------------------------------------------------------------
      read_kbd();
      delay(200);
//    -----------------------------------------------------------------
//    --- change used I2C address -------------------------------------
//    -----------------------------------------------------------------
      if (active_key == 0x0A)
      {
        i2c_t_change_add();
        while(active_key != 0xFF) read_kbd();
      }
//    -----------------------------------------------------------------
//    --- 2 / 4 decimal places ----------------------------------------
//    -----------------------------------------------------------------
      if (active_key == 0x02)
      {
        gv_format++;
        if (gv_format == 2) gv_format = 0;
        while(active_key != 0xFF) read_kbd();
      }
   }
   gv_refresh = 1;
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --- Change the last 3 bits of used I2C address for temp sensors ----
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void i2c_t_change_add(void)
{
  i2c_add++;
  if (bitRead(i2c_add,3) == 0) i2c_add = 0b01001000;
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------


// --------------------------------------------------------------------
// --------------------------------------------------------------------
// AT30TSE754 - update configuration register
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void t_change_res()
{
  temp_res++;
  
  if (temp_res > 3) temp_res = 0x00;
  v_config_msb = temp_res * 32;         //all other config bits are eq zero
  Wire.beginTransmission(i2c_add);
  Wire.write(0x01);
  Wire.write(v_config_msb);
  Wire.endTransmission(1);  
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// AT30TSE754 - read configuration register
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void read_t_config_reg()
{  
  Wire.beginTransmission(i2c_add);
  Wire.write(0x01);
  Wire.endTransmission(1);
  Wire.requestFrom(i2c_add, 1, 1);    //I2C address / 1 byte // stop
  v_config_msb = Wire.read();

  temp_res = 2 * bitRead(v_config_msb,6) + bitRead(v_config_msb,5);
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------


// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --- Write Temperature to LCD w/o conversion
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void w_temp_ub(byte i_col, byte i_row, byte i_sw2)
{
  int l_temp;
  
  lcd.setCursor(i_col, i_row);
  if (v_temp_lsb == 0xFF)
  {
    lcd.print("--       ");
  }
  else
  {
  if (bitRead(v_temp_msb,7) == 0)
  {
    lcd.print("+");
    w_print_dec1_nlz(v_temp_msb);           //Write value w/o leading zeros
    l_temp = (v_temp_lsb / 16) * 625;
    lcd.print(".");
    w_print_dec_wlz(l_temp);

    lcd.print(char(223));
    lcd.print("C");
    if (gv_format == 1 && i_sw2 == 0) lcd.print("  ");
  }
  else
  {
    w_hex(v_temp_msb);
    lcd.print("-");
    w_hex(v_temp_lsb);
    lcd.print("      ");
  }
  }
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------

// --------------------------------------------------------------------
// --------------------------------------------------------------------
void w_print_dec_wlz(int in_byte)
{
  if (gv_format == 0x00)
  {
    if(in_byte < 1000) lcd.print("0");
    if(in_byte <  100) lcd.print("0");
    if(in_byte <   10) lcd.print("0");
    lcd.print(in_byte);
  }
  else
  {
    in_byte = in_byte / 100;
    if(in_byte <   10) lcd.print("0");
    lcd.print(in_byte);
  }
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------


// --------------------------------------------------------------------
// --------------------------------------------------------------------
void w_print_dec01_wlz(byte in_byte)
{
  if(in_byte <  10) lcd.print("0");
  lcd.print(in_byte);
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------

// --------------------------------------------------------------------
// --------------------------------------------------------------------
void w_print_dec1_nlz(byte in_byte)
{
  if(in_byte <  10) lcd.print(" ");
  lcd.print(in_byte);
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void w_print_dec_nlz(byte in_byte)
{
  if(in_byte < 100) lcd.print(" ");
  if(in_byte <  10) lcd.print(" ");
  lcd.print(in_byte);
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void w_i2c_add(byte i_col, byte i_row)
{
  lcd.setCursor(i_col, i_row);
  lcd.print(bitRead(i2c_add,6));
  lcd.print(bitRead(i2c_add,5));
  lcd.print(bitRead(i2c_add,4));
  lcd.print(bitRead(i2c_add,3));
  lcd.print(" ");
  lcd.print(bitRead(i2c_add,2));
  lcd.print(bitRead(i2c_add,1));
  lcd.print(bitRead(i2c_add,0));
  lcd.print("x");
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------



// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void read_kbd()
{
   active_key = 0xFF;                 //no key is active
   in_kbd_7 = get_col(0b01111111);
   pcf8574_send(in_kbd_7);
   in_kbd_6 = get_col(0b10111111);
   pcf8574_send(in_kbd_6);
   in_kbd_5 = get_col(0b11011111);
   pcf8574_send(in_kbd_5);
   in_kbd_4 = get_col(0b11101111);
   pcf8574_send(in_kbd_4);
   get_col(0b11111111);
   pcf8574_send(0xFF);

   switch(in_kbd_7)
     {
      case 0b01111110:
         active_key = 0x01;         //key 1
         break;
      case 0b01111101:
         active_key = 0x04;         //key 4
         break;
      case 0b01111011:
         active_key = 0x07;         //key 7
         break;
      case 0b01110111:
         active_key = 0x0E;         //key *
         break;
     }

   switch(in_kbd_6)
     {
      case 0b10111110:
         active_key = 0x02;         //key 2
         break;
      case 0b10111101:
         active_key = 0x05;         //key 5
         break;
      case 0b10111011:
         active_key = 0x08;         //key 8
         break;
      case 0b10110111:
         active_key = 0x00;         //key 0
         break;
     }

   switch(in_kbd_5)
     {
      case 0b11011110:
         active_key = 0x03;         //key 3
         break;
      case 0b11011101:
         active_key = 0x06;         //key 6
         break;
      case 0b11011011:
         active_key = 0x09;         //key 9
         break;
      case 0b11010111:
         active_key = 0x0F;         //key #
         break;
     }

   switch(in_kbd_4)
     {
      case 0b11101110:
         active_key = 0x0A;         //key A
         break;
      case 0b11101101:
         active_key = 0x0B;         //key B
         break;
      case 0b11101011:
         active_key = 0x0C;         //key C
         break;
      case 0b11100111:
         active_key = 0x0D;         //key D
         break;
     }
   act_key_ascii = 0xFF;
   if (active_key != 0xFF)
     {
      act_key_ascii = active_key + 48;
      if (active_key > 0x09) act_key_ascii = active_key + 55;
     }
   if (act_key_ascii == 'E') act_key_ascii = '*';
   if (act_key_ascii == 'F') act_key_ascii = '#';
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
byte get_col(byte in_col)
{
   Wire.beginTransmission(0b00111111);      //PCF8574AN -> 0111 111x
   Wire.write(in_col);
   Wire.endTransmission(1);
   Wire.requestFrom(0b00111111, 1, 1);    //I2C address / 1 byte // stop
   return(Wire.read());
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------

// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void w_bin(byte iv_byte)
{
  lcd.print(bitRead(iv_byte,7));
  lcd.print(bitRead(iv_byte,6));
  lcd.print(bitRead(iv_byte,5));
  lcd.print(bitRead(iv_byte,4));
  lcd.print(bitRead(iv_byte,3));
  lcd.print(bitRead(iv_byte,2));
  lcd.print(bitRead(iv_byte,1));
  lcd.print(bitRead(iv_byte,0));
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------


// --------------------------------------------------------------------
// --------------------------------------------------------------------
// --------------------------------------------------------------------
void w_hex(byte iv_byte)
{
  if (iv_byte < 0x10) lcd.print("0");
  lcd.print(iv_byte, HEX);
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------

// --------------------------------------------------------------------
// --------------------------------------------------------------------
void pcf8574_send(byte iv_byte)
{
   Wire.beginTransmission(0b00100000);
   Wire.write(iv_byte);
   Wire.endTransmission(1);
}
// --------------------------------------------------------------------
// --------------------------------------------------------------------
